"""Check health of the installation."""

import logging

from .font import verify_fonts_bool
from .slack import check_env as slack_check_env
from .slack import check_send_text as slack_check_send_text

logger = logging.getLogger(__name__)


def main():
    successes = [slack_check_env()]
    successes.append(slack_check_send_text())
    successes.append(verify_fonts_bool())

    if all(successes):
        logger.info("")
        logger.info("💪 You are ready to go!")


if __name__ == "__main__":
    main()



import logging
import os

from ..send_only import send_text

logger = logging.getLogger(__name__)


def check_one_env(env_var, secret=False, secret_show_first_n=0):
    value = os.environ.get(env_var)
    if value is None:
        logger.error(f"😡 Please set the environment variable {env_var}.")
        return False

    if secret:
        if secret_show_first_n > 0:
            logger.info(
                f"✅ {env_var} is set to {value[:secret_show_first_n]}{'*' * (len(value) - secret_show_first_n)}"
            )
        else:
            logger.info(f"✅ {env_var} is set.")
    else:
        logger.info(f"✅ {env_var} is set to {value}")
    return True


def check_many_env(env_vars, secret=False, secret_show_first_n=0):
    ret = True
    for env_var in env_vars:
        if not check_one_env(env_var, secret, secret_show_first_n):
            ret = False
    return ret


def check_env():
    secrets_checked = check_many_env(["SLACK_BOT_TOKEN", "SLACK_APP_TOKEN"], True, 5)
    normal_checked = check_many_env(["SLACK_CHANNEL_ID"])

    return secrets_checked and normal_checked


def check_send_text():
    logger.info("🚀 Sending a test message to the Slack channel...")
    response = send_text("💪 Checking health of the Slack bot.")

    if response is None:
        logger.error("😡 The response is None.")
        return False

    if response["ok"]:
        logger.info("✅ The message was sent successfully.")
        return True

    logger.error(f"😡 The message was not sent successfully. Response: {response}")
    return False



import logging
from io import BytesIO, IOBase

import cairosvg
from matplotlib.figure import Figure
from PIL import Image
from slack_sdk.web import WebClient

from . import channel_id as default_channel_id
from . import client as default_client

logger = logging.getLogger(__name__)
warned = False


def warn_once():
    global warned
    if not warned:
        logger.warning(
            "You tried to send messages on Slack, but the token is not set. All messages will be ignored."
        )
        warned = True


def send_text(
    text: str,
    *,
    channel_id: str | None = None,
    client: WebClient | None = None,
):
    """
    Send a text message to the default client and channel (if not specified).
    """
    if client is None:
        client = default_client
    if client is None:
        warn_once()
        return None
    if channel_id is None:
        channel_id = default_channel_id
        assert channel_id is not None

    return client.chat_postMessage(channel=channel_id, text=text)


def send_text_as_file(
    *,
    filename: str,
    content: str,
    title: str,
    ensure_preview: bool = False,
    initial_comment: str | None = None,
    channel_id: str | None = None,
    client: WebClient | None = None,
):
    """
    Send a text as a file to the default client and channel (if not specified).

    Slack is stupid and having special characters in the file makes it think it's a binary file.
    If ensure_preview is True, then we append `""" "dear-viewer" """` at the beginning of the file so Slack thinks
    it's a python file and will preview it.

    Note:
        `ensure_preview` is a hacky workaround and may not work in the future.

    Issues:
        `ensure_preview` only works with text-ish files (txt, html, etc.) and not with other files (pdf, png, svg).
        You can set the `filename` to be like "file.html" for svg files with the `ensure_preview` flag.

    Args:
        filename:
        content:
        title:
        ensure_preview: If True, append `""" "dear-viewer" """` at the beginning of the file to ensure that
                        Slack will preview it.
        initial_comment:
        channel_id:
        client:
    """
    if client is None:
        client = default_client
    if client is None:
        warn_once()
        return None
    if channel_id is None:
        channel_id = default_channel_id
        assert channel_id is not None

    if ensure_preview:
        content = '""""dear-viewer""""\n' + content

    return client.files_upload_v2(
        filename=filename,
        content=content,
        title=title,
        channel=channel_id,
        initial_comment=initial_comment,
    )


def send_divider(
    *,
    channel_id: str | None = None,
    client: WebClient | None = None,
):
    """
    Send a divider block to the default client and channel (if not specified).
    """
    if client is None:
        client = default_client
    if client is None:
        warn_once()
        return None
    if channel_id is None:
        channel_id = default_channel_id
        assert channel_id is not None

    return client.chat_postMessage(
        channel=channel_id,
        text="Divider",
        blocks=[
            {"type": "divider"},
        ],
    )


def send_file(
    *,
    filename: str,
    file: str | bytes | IOBase,
    title: str,
    snippet_type: str | None = None,
    initial_comment: str | None = None,
    channel_id: str | None = None,
    client: WebClient | None = None,
):
    """
    Send a file to the default client and channel (if not specified).
    """
    if client is None:
        client = default_client
    if client is None:
        warn_once()
        return None
    if channel_id is None:
        channel_id = default_channel_id
        assert channel_id is not None

    return client.files_upload_v2(
        filename=filename,
        file=file,
        title=title,
        channel=channel_id,
        snippet_type=snippet_type,
        initial_comment=initial_comment,
    )


def send_matplotlib_fig(
    filename: str,
    fig: Figure,
    title: str,
    initial_comment: str | None = None,
    channel_id: str | None = None,
    client: WebClient | None = None,
):
    """
    Send a matplotlib figure as a PDF file to the default client and channel (if not specified).
    """
    if client is None:
        client = default_client
    if client is None:
        warn_once()
        return None
    if channel_id is None:
        channel_id = default_channel_id
        assert channel_id is not None

    buf = BytesIO()
    fig.savefig(buf, format="pdf")
    buf.seek(0)
    return send_file(
        filename=filename,
        file=buf,
        title=title,
        initial_comment=initial_comment,
        channel_id=channel_id,
        client=client,
    )


def send_pil_image(
    filename: str,
    image: Image.Image,
    title: str,
    initial_comment: str | None = None,
    channel_id: str | None = None,
    client: WebClient | None = None,
):
    """
    Send a PIL image as a PNG file to the default client and channel (if not specified).
    """
    if client is None:
        client = default_client
    if client is None:
        warn_once()
        return None
    if channel_id is None:
        channel_id = default_channel_id
        assert channel_id is not None

    buf = BytesIO()
    image.save(buf, format="png")
    buf.seek(0)
    return send_file(
        filename=filename,
        file=buf,
        title=title,
        initial_comment=initial_comment,
        channel_id=channel_id,
        client=client,
    )


def send_svg_as_pdf(
    filename: str,
    svg_file: str | bytes | IOBase,
    title: str,
    initial_comment: str | None = None,
    channel_id: str | None = None,
    client: WebClient | None = None,
):
    """
    Send an SVG file as a PDF file.

    Slack does not support previewing SVG files, so we convert it to PDF.

    Issues:
        The cairosvg library does not have good support for fonts, and if you `export_svg` from `rich.console`
        then it will have font alignment issues, even if you install the Fira Code font on your system.
    """
    if client is None:
        client = default_client
    if client is None:
        warn_once()
        return None
    if channel_id is None:
        channel_id = default_channel_id
        assert channel_id is not None

    pdf_buf = BytesIO()

    if isinstance(svg_file, str | bytes):
        cairosvg.svg2pdf(bytestring=svg_file, write_to=pdf_buf)
    elif isinstance(svg_file, IOBase):
        cairosvg.svg2pdf(file_obj=svg_file, write_to=pdf_buf)
    else:
        raise ValueError(f"Unsupported type {type(svg_file)}")

    pdf_buf.seek(0)

    return client.files_upload_v2(
        filename=filename,
        file=pdf_buf,
        title=title,
        channel=channel_id,
        initial_comment=initial_comment,
    )



import inspect
import logging
import os
from datetime import datetime, timezone
from os import PathLike
from pathlib import Path

from rich.logging import RichHandler

from slack_helpers import __version__

logger = logging.getLogger(__name__)


def setup_logging(
    console_level: int = logging.INFO,
    log_dir: str | PathLike | None = None,
    output_files: list[str] | None = None,
    file_levels: list[int] | None = None,
):
    """
    Setup logging with RichHandler and FileHandler.

    You should call this function at the beginning of your script.

    Args:
        console_level: Logging level for console. Defaults to INFO or env var MLPROJECT_LOG_LEVEL.
        log_dir: Directory to save log files. If None, do not save log files.
        output_files: List of output file paths, relative to log_dir. If None, use default.
        file_levels: List of logging levels for each output file. If None, use default.
    """
    if log_dir is None:
        assert output_files is None, "output_files must be None if log_dir is None"
        assert file_levels is None, "file_levels must be None if log_dir is None"

        output_files = []
        file_levels = []
    else:
        log_dir = Path(log_dir)

        if output_files is None:
            output_files = ["{date:%Y%m%d-%H%M%S}-{name}-{levelname}-{version}.log"]
        if file_levels is None:
            file_levels = [logging.INFO]

    assert len(output_files) == len(
        file_levels
    ), "output_files and file_levels must have the same length"

    # NOTE: Initialise with NOTSET level and null device, and add stream handler separately.
    # This way, the root logging level is NOTSET (log all), and we can customise each handler's behaviour.
    # If we set the level during the initialisation, it will affect to ALL streams,
    # so the file stream cannot be more verbose (lower level) than the console stream.
    logging.basicConfig(
        format="",
        level=logging.NOTSET,
        stream=open(os.devnull, "w"),  # noqa: SIM115
    )

    # If you want to suppress logs from other modules, set their level to WARNING or higher
    # logging.getLogger('some_package.utils').setLevel(logging.WARNING)

    console_handler = RichHandler(
        level=console_level,
        show_time=True,
        show_level=True,
        show_path=True,
        rich_tracebacks=True,
        tracebacks_show_locals=True,
    )
    console_format = logging.Formatter(
        fmt="%(name)s - %(message)s",
        datefmt="%m/%d %H:%M:%S",
    )
    console_handler.setFormatter(console_format)

    f_format = logging.Formatter(
        fmt="%(asctime)s - %(name)s: %(lineno)4d - %(levelname)s - %(message)s",
        datefmt="%y/%m/%d %H:%M:%S",
    )

    function_caller_module = inspect.getmodule(inspect.stack()[1][0])
    if function_caller_module is None:
        name_or_path = "unknown"
    elif function_caller_module.__name__ == "__main__":
        if function_caller_module.__file__ is None:
            name_or_path = function_caller_module.__name__
        else:
            name_or_path = function_caller_module.__file__.replace("/", ".")
            # Remove .py extension
            name_or_path = Path(name_or_path).with_suffix("")

    else:
        name_or_path = function_caller_module.__name__

    log_path_map = {
        "name": name_or_path,
        "version": __version__,
        "date": datetime.now(timezone.utc),
    }

    root_logger = logging.getLogger()
    root_logger.addHandler(console_handler)

    if log_dir is not None:
        log_paths = []
        for output_file, file_level in zip(output_files, file_levels, strict=True):
            log_path_map["levelname"] = logging._levelToName[file_level]
            log_path = log_dir / output_file.format_map(log_path_map)
            log_path.parent.mkdir(parents=True, exist_ok=True)

            f_handler = logging.FileHandler(log_path)
            f_handler.setLevel(file_level)
            f_handler.setFormatter(f_format)

            # Add handlers to the logger
            root_logger.addHandler(f_handler)

        for log_path in log_paths:
            logger.info(f"Logging to {log_path}")


def main():
    logger.info("Hello, world!")
    raise Exception("Test exception")


if __name__ == "__main__":
    try:
        setup_logging()
        main()
    except Exception:
        logger.exception("Exception occurred")



import logging
import subprocess

logger = logging.getLogger(__name__)


class FontNotInstalledError(Exception):
    pass


def verify_fonts_installed():
    """
    This function verifies that the FiraCode font is installed because Rich Console.export_svg() uses it.

    Actually, the svg itself doesn't need the font but when we convert it to PDF using cairosvg, the font is needed.
    """
    cmd = ["fc-match", "FiraCode:style=Regular"]
    output = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()[0]

    if not output.decode("utf-8").startswith("FiraCode-Regular"):
        raise FontNotInstalledError("FiraCode")

    cmd = ["fc-match", "FiraCode:style=Bold"]
    output = subprocess.Popen(cmd, stdout=subprocess.PIPE).communicate()[0]

    if not output.decode("utf-8").startswith("FiraCode-Bold"):
        raise FontNotInstalledError("FiraCode")


def verify_fonts_bool():
    try:
        verify_fonts_installed()
        logger.info("✅ FiraCode font is installed.")
        return True
    except FontNotInstalledError:
        logger.error("😡 FiraCode font is not installed.")
        return False


if __name__ == "__main__":
    verify_fonts_installed()



# ruff: noqa: T201
import typer

app = typer.Typer(
    no_args_is_help=True, context_settings={"help_option_names": ["-h", "--help"]}
)


def version_callback(value: bool):
    if value:
        from .. import __version__

        print(__version__)
        raise typer.Exit()


@app.callback()
def common(
    ctx: typer.Context,
    version: bool = typer.Option(
        None, "-v", "--version", callback=version_callback, help="Show version"
    ),
):
    pass


@app.command()
def health():
    from ..health import main as health_main
    from ..utils.log import setup_logging

    setup_logging()
    health_main()


def main():
    app()


if __name__ == "__main__":
    main()



import os

from slack_bolt import App

from . import _version

__version__ = _version.get_versions()["version"]


bot_token = os.environ.get("SLACK_BOT_TOKEN", None)

if bot_token is not None:
    app = App(token=bot_token)
    client = app.client
    channel_id = os.environ["SLACK_CHANNEL_ID"]
else:
    app = None
    client = None
    channel_id = None



"""
This module is used to start the Slack app in interactive mode.

Examples:
    >>> from slack_helpers import app
    >>> from slack_helpers.interactive import handler
    >>> @app.message("hello")  # doctest: +SKIP
    ... def hello_message(body: dict[str, Any], say):
    ...     logger.info("Received a hello command")
    ...     logger.info(f"body: {pformat(body)}")
    ...     say(text=f"Hi <@{body['event']['user']}>!")

    >>> handler.start()  # doctest: +SKIP
"""

import os

from slack_bolt.adapter.socket_mode import SocketModeHandler

from . import app, bot_token

app_token = os.environ.get("SLACK_APP_TOKEN", None)

if bot_token is None:
    handler = None
else:
    assert app_token is not None, "SLACK_APP_TOKEN environment variable is required"
    assert app is not None, "SLACK_BOT_TOKEN environment variable is required"
    handler = SocketModeHandler(app, app_token)



import os
from collections.abc import Iterable
from typing import Any, Literal

from ansi2html import Ansi2HTMLConverter
from rich.console import Console
from rich.terminal_theme import MONOKAI
from rich.traceback import Traceback

CONSOLE_WIDTH = 130


def rich_print_to_string(objs: Iterable[Any] | Any) -> str:
    """Convert a rich console print output to a string."""
    console = Console(
        width=CONSOLE_WIDTH,
        no_color=True,
        highlight=False,
        record=True,
        file=open(os.devnull, "w"),  # noqa: SIM115
    )

    if not isinstance(objs, Iterable):
        objs = [objs]
    for obj in objs:
        console.print(obj)

    return console.export_text()


def rich_print_to_html(
    objs: Iterable[Any] | Any,
    backend: Literal["ansi2html", "rich"] = "ansi2html",
) -> str:
    """
    Convert a rich console print output to an html string.

    rich backend can be prettier with the following caveats:
    1. It produces a larger file size (2x)
    2. It isn't as easy to read as a plain HTML code due to alignment and many html tags.
    3. Slack doesn't understand the HTML code produced by `console.export_html()` and treat it as a binary file.

    Args:
        objs: The objects to print. Possibly but not limited to rich objects.
        backend: "ansi2html" or "rich". Defaults to "ansi2html".
    """
    assert backend in ("ansi2html", "rich")

    console = Console(width=CONSOLE_WIDTH, record=True, file=open(os.devnull, "w"))  # noqa: SIM115

    if not isinstance(objs, Iterable):
        objs = [objs]
    for obj in objs:
        console.print(obj)

    if backend == "rich":
        return console.export_html(theme=MONOKAI)

    tb_ansi = console.export_text(styles=True)  # text with ansi color codes
    return Ansi2HTMLConverter().convert(tb_ansi)


def rich_print_to_svg(objs: Iterable[Any] | Any, title: str) -> str:
    """Convert a rich console print to an svg string."""
    console = Console(width=CONSOLE_WIDTH, record=True, file=open(os.devnull, "w"))  # noqa: SIM115

    if not isinstance(objs, Iterable):
        objs = [objs]
    for obj in objs:
        console.print(obj)

    return console.export_svg(title=title)


def rich_traceback_to_string(tb: Traceback) -> str:
    """
    Convert a rich traceback to a string.

    Examples:
        >>> try:   # doctest: +ELLIPSIS
        ...    1 / 0
        ... except ZeroDivisionError:
        ...     import rich.traceback
        ...     tb = rich.traceback.Traceback()
        ...     print(rich_traceback_to_string(tb))
        ╭─────────────────────────────── Traceback (most recent call last) ────────────────────────────────╮
        ...
        ╰──────────────────────────────────────────────────────────────────────────────────────────────────╯
        ZeroDivisionError: division by zero
        <BLANKLINE>
    """
    return rich_print_to_string([tb])


def rich_traceback_to_html(
    tb: Traceback, backend: Literal["ansi2html", "rich"] = "ansi2html"
) -> str:
    """
    Convert a rich traceback to an html string.

    Examples:
        >>> try:   # doctest: +ELLIPSIS
        ...    1 / 0
        ... except ZeroDivisionError:
        ...     import rich.traceback
        ...     tb = rich.traceback.Traceback()
        ...     print(rich_traceback_to_html(tb))
        <!DOCTYPE HTML ...
        </html>
        <BLANKLINE>
    """
    return rich_print_to_html([tb], backend=backend)


def rich_traceback_to_svg(tb: Traceback, title: str) -> str:
    """
    Convert a rich traceback to an svg string.

    Examples:
        >>> try:   # doctest: +ELLIPSIS
        ...    1 / 0
        ... except ZeroDivisionError:
        ...     import rich.traceback
        ...     tb = rich.traceback.Traceback()
        ...     print(rich_traceback_to_svg(tb, "title"))
        <svg class="rich-terminal" ...
        </svg>
        <BLANKLINE>
    """
    return rich_print_to_svg([tb], title)
